@@ -13,6 +13,7 @@ import cStringIO
 import email
 import email.Errors
 import email.Utils
+import email.Parser
 from email.Generator import Generator
 
 from exceptions import *
@@ -79,27 +80,27 @@ class Message(object):
         self.received_from = None
         self.received_with = None
         self.__raw = None
-
+        parser = email.Parser.HeaderParser(strict=False)
+        
         # Message is instantiated with fromlines for POP3, fromstring for
         # IMAP (both of which can be badly-corrupted or invalid, i.e. spam,
         # MS worms, etc).  It's instantiated with fromfile for the output
         # of filters, etc, which should be saner.
         if fromlines:
             try:
-                self.__msg = email.message_from_string(os.linesep.join(
-                    fromlines), strict=False)
+                self.__msg = parser.parsestr(os.linesep.join(fromlines))
             except email.Errors.MessageError, o:
                 self.__msg = corrupt_message(o, fromlines=fromlines)
             self.__raw = os.linesep.join(fromlines)
         elif fromstring:
             try:
-                self.__msg = email.message_from_string(fromstring, strict=False)
+                self.__msg = parser.parsestr(fromstring)
             except email.Errors.MessageError, o:
                 self.__msg = corrupt_message(o, fromstring=fromstring)
             self.__raw = fromstring
         elif fromfile:
             try:
-                self.__msg = email.message_from_file(fromfile, strict=False)
+                self.__msg = parser.parse(fromfile)
             except email.Errors.MessageError, o:
                 # Shouldn't happen
                 self.__msg = corrupt_message(o, fromstring=fromfile.read())
@@ -135,8 +136,9 @@ class Message(object):
             f.write('From %s %s' % (mbox_from_escape(self.sender),
                 time.asctime()) + os.linesep)
         # Write the Return-Path: header
-        f.write(format_header('Return-Path', self.sender))
-        # Maybe remove previous Return-Path: header fields?
+        f.write(format_header('Return-Path', '<%s>' % self.sender))
+        # Remove previous Return-Path: header fields.
+        del self.__msg['Return-Path']
         if delivered_to:
             f.write(format_header('Delivered-To', self.recipient or 'unknown'))
         if received:
